'' =================================================================================================
''
''   File....... jm_wav_player_demo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 19 MAY 2013
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con 

  #(-1), IS_ON, IS_OFF

  #0, LSBFIRST, MSBFIRST
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  audio : "jm_wav_player"  
  term  : "fullduplexserial64"


var


dat

  {
    Parameters for audio setup

    L/R channel outputs        (-1 if not used)
    SD card connections        (4 pins)
    Dither for stereo_dacs cog
  }

  AudioSetup            long    26, 27                          ' for Propeller Activity Board
                        long    22, 23, 24, 25
                        long    5 

  AudioSetupQMP         long    22, 23                          ' for Ray Allen's Quick Media Player
                        long    27, 26, 25, 24
                        long    3

  
  AudioFile             byte    "spalding.wav", 0

  
  Menu                  byte    "-------------------", CR
                        byte    " Audio Player Demo ", CR
                        byte    "-------------------", CR
                        byte    CR
                        byte    "[R] Report         ", CR                          
                        byte    "[P] Play normal    ", CR
                        byte    "[F] Play fast      ", CR
                        byte    "[S] Play slow      ", CR
                        byte    "[X] Stop playback  ", CR
                        byte    "[A] Auto-pan demo  ", CR
                        byte    0 

                        
  Report                byte    "--------------------------", CR
                        byte    " Atrributes:              ", CR
                        byte    "--------------------------", CR
                        byte    CR
                        byte    "Channels......            ", CR
                        byte    "Sample Rate...            ", CR
                        byte    "Duration......            ", CR
                        byte    CR
                        byte    "Press a key."
                        byte    0
  

pub main | c

  audio.start(@AudioSetup)                                      ' start wav player

  term.start(RX1, TX1, %0000, 115_200)                          ' start serial io for terminal
  pause(10)

  repeat
    term.rxflush                                                ' clear keyboard buffer
    term.tx(CLS)                                                ' clear screen
    term.str(@Menu)                                             ' display menu

    c := ucase(term.rx)                                         ' wait for user input

    case c
      "R":
        show_report(@AudioFile)                                 ' show audio file attributes
    
      "P":
        audio.set_speed(100)                                    ' reset playback speed
        audio.play(@AudioFile, 1)
        
      "F":
        audio.set_speed(125)                                    ' speed-up playback
        audio.play(@AudioFile, 1)

      "S":
        audio.set_speed(75)                                     ' slow down playback
        audio.play(@AudioFile, 1)

      "X":
        audio.kill
        
      "A":
        auto_pan(@AudioFile, 100, 0)                            ' pan audio while playing


pub auto_pan(p_filename, lstart, rstart) | check, span, delay, direction, t

'' Demonstrates real-time volume control will file
'' -- pan direction based on lstart and rstart (0 to 100)

  check := audio.get_header(p_filename)                         ' load header from file
  if (check < 0)                                                ' if any error
    term.tx(CLS)
    term.str(string("Cannot open "))                            '  alert user
    term.str(p_filename)
    term.rxflush
    term.rx

  span := ||(lstart - rstart)                                   ' span of pan effect
  delay := audio.duration * 10 / span * MS_001                  ' delay between updates              

  if (lstart > rstart)
    direction := 1
  else
    direction := 0

  audio.set_speed(100)                                          ' reset playback speed
  audio.set_volume(lstart, rstart)                              ' set starting volume
  audio.play(@AudioFile, 1)                                     ' start the file

  term.rxflush                                                  ' clear key buffer before starting

  t := cnt                                                      ' sync delay timer
  repeat while (audio.playing)
    waitcnt(t += delay)
    if (direction)
      audio.set_volume(--lstart, ++rstart)                      ' pan left to right
    else
      audio.set_volume(++lstart, --rstart)                      ' pan right to left

    check := ucase(term.rxcheck)                                ' check for stop command
    if (check == "X")
      audio.kill
      quit

  audio.set_volume(100, 100)                                    ' reset volume levels 
    

pub show_report(p_filename) | check, secs

'' Opens file (if possible) and shows basic parameters

  term.tx(CLS)

  check := audio.get_header(p_filename)                         ' load header from file
  if (check < 0)                                                ' if any error
    term.str(string("Cannot open "))                            '  alert user
    term.str(p_filename)
    term.rxflush
    term.rx

  term.str(@Report)

  move_to(13, 1)
  term.str(p_filename)                                          ' display file name

  move_to(15, 4)
  term.dec(audio.channels)                                      ' display # of channels

  move_to(15, 5)
  term.dec(audio.sample_rate)                                   ' display audio sample rate

  move_to(15, 6)
  secs := audio.duration                                        ' duration in 0.01s units
  term.dec(secs / 100)
  term.tx(".")
  term.rjdec(secs // 100, 2, "0")

  term.rxflush
  term.rx


pub ucase(c)

'' If c is lowercase, convert to uppercase

  if ((c => "a") and (c =< "z"))
    c -= 32

  return c
  

pub move_to(x, y)

'' Move cursor for PST

  term.tx(GOTOXY)
  term.tx(x)
  term.tx(y)
  

con

  { --------------- }
  {                 }
  {   B A S I C S   }
  {                 }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}